@@ -40,16 +40,14 @@ sub setup_grammar {
 			map{ $_, 1} split " ", get_data_section("midish_commands")
 	};
 
-	# print remove_spaces("bulwinkle is a...");
-
 }
-{
-my %exclude_from_undo_buffer = map{ $_ => 1} 
-		qw(tag commit branch br new_branch nbr load save get restore);
 sub process_line {
 	logsub("&process_line");
 	no warnings 'uninitialized';
 	my ($user_input) = @_;
+	# convert hyphenated commands to underscore form
+	while( my ($from, $to) = each %{$text->{hyphenated_commands}})
+	{ $user_input =~ s/$from/$to/g }
 	logpkg(__FILE__,__LINE__,'debug',"user input: $user_input");
 	if (defined $user_input and $user_input !~ /^\s*$/) {
 		$text->{term}->addhistory($user_input) 
@@ -61,37 +59,42 @@ sub process_line {
 					:  midish_command($user_input);	
 		}
 		else {
+			my $context = context();
 			my $success = process_command( $user_input );
-				
-			push @{$project->{undo_buffer}}, 
-
-			{
-				context => context(),
-				command => $user_input,
-			#	commit 	=> $commit 
+			my $command_stamp = { context => $context, 
+								  command => $user_input };
+			push(@{$project->{command_buffer}}, $command_stamp);
+			
+			if ( 		$config->{autosave} eq 'undo'
+					and $config->{use_git} 
+					and $project->{name}
+					and $project->{repo}
+					and ! engine_running() 
+			){
+				local $quiet = 1;
+				Audio::Nama::ChainSetup::remove_temporary_tracks();
+				autosave() unless $config->{opts}->{R};
+				reconfigure_engine(); # quietly, avoiding noisy reconfig below
 			}
-
-				unless ! $success 
-					   or $user_input =~ /^\s*([a-z_]+)/
-						and $exclude_from_undo_buffer{$1};
-			autosave() if $config->{use_git} and $config->{autosave} eq 'undo';
 			reconfigure_engine();
-				#or eval_iam('cs-connected') 
-				#and remove_latency_ops() 
-				#and calculate_and_adjust_latency();
 		}
-		revise_prompt( $mode->{midish_terminal} ? "Midish > " : prompt());
+		# reset current track to Master if it is
+		# undefined, or the track has been removed
+		# from the index
+		$this_track = $tn{Master} if ! $this_track or
+			(ref $this_track and ! $tn{$this_track->name});
+		setup_hotkeys() if $config->{hotkeys_always};
 	}
-}
+	revise_prompt( $mode->{midish_terminal} and "Midish > " );
+	my $output = delete $text->{output_buffer};
 }
 sub context {
 	my $context = {};
 	$context->{track} = $this_track->name;
 	$context->{bus}   = $this_bus;
-	$context->{op}    = $this_op;
+	$context->{op}    = $this_track->op;
 	$context
 }
-	
 sub process_command {
 	state $total_effects_count;
 	my $input = shift;
@@ -106,7 +109,7 @@ sub process_command {
 		logpkg(__FILE__,__LINE__,'debug',"input: $input");
 		$text->{parser}->meta(\$input) or do
 		{
-			print("bad command: $input_was\n"); 
+			throw("bad command: $input_was\n"); 
 			$was_error++;
 			system($config->{beep_command}) if $config->{beep_command};
 			last;
@@ -120,9 +123,10 @@ sub process_command {
 	set_current_bus();
 	# select chain operator if appropriate
 	no warnings 'uninitialized';
-	if ($this_op and $this_track->n eq chain($this_op)){
+	my $FX = fxn($this_track->op);
+	if ($FX and $this_track->n eq $FX->chain){
 		eval_iam("c-select ".$this_track->n);
-		eval_iam("cop-select ".  ecasound_effect_index($this_op));
+		eval_iam("cop-select ".  $FX->ecasound_effect_index);
 	}
 
 	my $result = check_fx_consistency();
@@ -132,7 +136,8 @@ sub process_command {
 	my $current_count= 0;
 	map{ $current_count++ } keys %{$fx->{applied}};
 	if ($current_count < $total_effects_count){
-		say "Total effects count: $current_count, change: ",$current_count - $total_effects_count; 
+		pager("Total effects count: $current_count, change: ",
+			$current_count - $total_effects_count);
 		$total_effects_count = $current_count;
 	}
 	# return true on complete success
@@ -142,7 +147,6 @@ sub process_command {
 		
 }
 sub do_user_command {
-	#say "args: @_";
 	my($cmd, @args) = @_;
 	$text->{user_command}->{$cmd}->(@args);
 }	
@@ -164,7 +168,7 @@ sub do_script {
 			if(-e $filename){}
 			else{ $filename = join_path(project_root(),$name) }
 		}
-		-e $filename or say("$filename: file not found. Skipping"), return;
+		-e $filename or throw("$filename: file not found. Skipping"), return;
 		$script = read_file($filename)
 	}
 	my @lines = split "\n",$script;
@@ -183,7 +187,7 @@ sub dump_all {
 }
 
 
-sub leading_track_spec {
+sub user_set_current_track {
 	my $cmd = shift;
 	if( my $track = $tn{$cmd} || $ti{$cmd} ){
 		logpkg(__FILE__,__LINE__,'debug',"Selecting track ",$track->name);
@@ -219,14 +223,16 @@ sub eval_perl {
 	my $code = shift;
 	map{ $code =~ s/(^|[^A-Za-z])::$_/$1$namespace_root\::$_/ } @namespace_abbreviations; # SKIP_PREPROC
 	my $err;
+	undef $text->{eval_result};
 	my @result = eval $code;
 	if ($@){
-		print( "Perl command failed: \ncode: $code\nerror: $@");
+		throw( "Perl command failed: \ncode: $code\nerror: $@");
 		undef $@;
 	}
 	else { 
 		no warnings 'uninitialized';
 		@result = map{ dumper($_) } @result;
+		$text->{eval_result} = join " ", @result;
 		pager(join "\n", @result) 
 	}	
 }
@@ -245,7 +251,7 @@ sub show_versions {
 
 
 sub show_send { "Send: ". $this_track->send_id. $/ 
-					if $this_track->rec_status ne 'OFF'
+					if $this_track->rec_status ne OFF
 						and $this_track->send_id
 }
 
@@ -262,44 +268,59 @@ sub list_effects {
 
 sub list_effect {
 	my $op_id = shift;
-	my $name = name($op_id);
-	$name .= q(, bypassed) if bypassed($op_id);
-	($op_id eq $this_op ? '*' : '') . "$op_id ($name)";
+	my $FX = fxn($op_id);
+	my $name = $FX->name;
+	$name .= q(, bypassed) if $FX->bypassed;
+	($op_id eq $this_track->op ? '*' : '') . "$op_id ($name)";
 }
 
-
 sub show_effect {
  	my $op_id = shift;
-	my @lines;
-	my @params;
- 	my $i = fxindex($op_id);
-	my $name = name($op_id);
-	my $ladspa_id = $fx_cache->{ladspa_label_to_unique_id}->{type($op_id)} ;
-	$name .= " ($ladspa_id)" if $ladspa_id;
-	$name .= " (bypassed)" if bypassed($op_id);
-	$name .= "\n";
- 	push @lines, "$op_id: $name";
+	my $with_track = shift;
+	my $FX = fxn($op_id);
+	return unless $FX;
+	my @lines = $FX->nameline;
+	#EQ: GVerb, gverb, 1216, bypassed, famp5, neap
+ 	my $i = $FX->registry_index;
 	my @pnames = @{$fx_cache->{registry}->[ $i ]->{params}};
 	{
 	no warnings 'uninitialized';
-	map
-	{ 
-		my $name = $pnames[$_]->{name};
-		$name .= " (read-only)" if $pnames[$_]->{dir} eq 'output';
-		push @lines, "    ".($_+1).q(. ) . $name . ": ".  params($op_id)->[$_] . "\n";
-	} (0..scalar @pnames - 1);
+	map { push @lines, parameter_info_padded($op_id, $_) } (0..scalar @pnames - 1) 
 	}
 	map
-	{ 	push @lines,
-	 	"    ".($_+1).": ".  $fx->{params}->{$op_id}->[$_] . "\n";
+	{ 	push @lines, parameter_info_padded($op_id, $_) 
+	 	
 	} (scalar @pnames .. (scalar @{$fx->{params}->{$op_id}} - 1)  )
 		if scalar @{$fx->{params}->{$op_id}} - scalar @pnames - 1; 
-	#push @lines, join("; ", @params) . "\n";
 	@lines
 }
+sub extended_name {
+	no warnings 'uninitialized';
+	my $op_id = shift;
+	my $FX = fxn($op_id);
+	return unless $FX;
+	my $name = $FX->name;
+	my $ladspa_id = $fx_cache->{ladspa_label_to_unique_id}->{$FX->type};
+	$name .= " ($ladspa_id)" if $ladspa_id;
+	$name .= " (bypassed)" if $FX->bypassed;
+	$name;
+}
+sub parameter_info {
+	no warnings 'uninitialized';
+	my ($op_id, $parameter) = @_;  # zero based
+	my $FX = fxn($op_id);
+	return unless $FX;
+	my $entry = $FX->about->{params}->[$parameter];
+	my $name = $entry->{name};
+	$name .= " (read-only)" if $entry->{dir} eq 'output';
+	($parameter+1).q(. ) . $name . ": ".  $FX->params->[$parameter];
+}
+sub parameter_info_padded {
+	" "x 4 . parameter_info(@_) . "\n";
+}
 sub named_effects_list {
 	my @ops = @_;
-	join("\n", map{ "$_ (" . Audio::Nama::name($_). ")" } @ops), "\n";
+	join("\n", map{ "$_ (" . fxn($_)->name. ")" } @ops), "\n";
 }
  
 sub show_modifiers {
@@ -308,18 +329,18 @@ sub show_modifiers {
 }
 sub show_region {
 	my $t = $Audio::Nama::this_track;
-	return unless $t->rec_status eq 'MON';
+	return unless $t->rec_status eq PLAY;
 	my @lines;
 	push @lines,join " ",
-		"Length:",time2($t->adjusted_length),"\n";
+		"Length:",time2($t->shifted_length),"\n";
 	$t->playat and push @lines,join " ",
-		"Play at:",time2($t->adjusted_playat_time),
+		"Play at:",time2($t->shifted_playat_time),
 		join($t->playat, qw[ ( ) ])."\n";
 	$t->region_start and push @lines,join " ",
-		"Region start:",time2($t->adjusted_region_start_time),
+		"Region start:",time2($t->shifted_region_start_time),
 		join($t->region_start, qw[ ( ) ])."\n";
 	$t->region_end and push @lines,join " ",
-		"Region end:",time2($t->adjusted_region_end_time),
+		"Region end:",time2($t->shifted_region_end_time),
 		join($t->region_end, qw[ ( ) ])."\n";
 	return(join "", @lines);
 }
@@ -329,17 +350,17 @@ sub time2 {
 	dn($n,3),"/",colonize(int ($n + 0.5));
 }
 sub show_status {
-	print "\n";
 	package Audio::Nama;
+	my @output;
 	my @modes;
 	push @modes, $mode->{preview} if $mode->{preview};
-	push @modes, "master" if $mode->{mastering};
+	push @modes, "master" if $mode->mastering;
 	push @modes, "edit"   if Audio::Nama::edit_mode();
-	push @modes, "offset run" if Audio::Nama::offset_run_mode();
-	say   "Modes settings:   ", join(", ", @modes) if @modes;
+	push @modes, "offset run" if Audio::Nama::is_offset_run_mode();
+	push @output, "Modes settings:   ", join(", ", @modes), $/ if @modes;
 	my @actions;
 	push @actions, "record" if grep{ ! /Mixdown/ } Audio::Nama::ChainSetup::really_recording();
-	push @actions, "playback" if grep { $_->rec_status eq 'MON' } 
+	push @actions, "playback" if grep { $_->rec_status eq PLAY } 
 		map{ $tn{$_} } $bn{Main}->tracks, q(Mixdown);
 
 	# We only check Main bus for playback. 
@@ -348,18 +369,16 @@ sub show_status {
 	# tracks are set to REC (with rec-to-file disabled)
 	
 	
-	push @actions, "mixdown" if $tn{Mixdown}->rec_status eq 'REC';
-	say "Pending actions:  ", join(", ", @actions) if @actions;
-	say "Main bus allows:  ", $bn{Main}->allows, " track status";
-	say "Main bus version: ",$bn{Main}->version if $bn{Main}->version;
-	say "Setup length is:  ", Audio::Nama::heuristic_time($setup->{audio_length}); 
-	say "Run time limit:   ", Audio::Nama::heuristic_time($setup->{runtime_limit})
+	push @actions, "mixdown" if $tn{Mixdown}->rec_status eq REC;
+	push @output, "Pending actions:  ", join(", ", @actions), $/ if @actions;
+	push @output, "Main bus version: ",$bn{Main}->version, $/ if $bn{Main}->version;
+	push @output, "Setup length is:  ", Audio::Nama::heuristic_time($setup->{audio_length}), $/; 
+	push @output, "Run time limit:   ", Audio::Nama::heuristic_time($setup->{runtime_limit}), $/
       if $setup->{runtime_limit};
-		
 }
 sub placeholder { 
 	my $val = shift;
-	return $val if defined $val;
+	return $val if defined $val and $val !~ /^\s*$/;
 	$config->{use_placeholders} ? q(--) : q() 
 }
 
@@ -373,14 +392,14 @@ sub show_inserts {
 }
 
 $text->{format_top} = <<TOP;
- No. Name            Ver  Set  Stat       Source       Bus         Vol  Pan
-=============================================================================
+ No. Name            Status     Source            Destination   Vol   Pan
+=========================================================================
 TOP
 
 $text->{format_divider} = '-' x 77 . "\n";
 
 my $format_picture = <<PICTURE;
-@>>  @<<<<<<<<<<<<<< @>>  @<<  @||||  @|||||||||||||   @<<<<<<<<<  @>>  @>> 
+@>>  @<<<<<<<<<<<<<< @<<<<<<<<< @<<<<<<<<<<<<<<<< @<<<<<<<<<<< @>>>  @>>>
 PICTURE
 
 sub show_tracks_section {
@@ -390,11 +409,9 @@ sub show_tracks_section {
     map {   formline $format_picture, 
             $_->n,
             $_->name,
-            placeholder( $_->current_version || undef ),
-			lc $_->rw,
             $_->rec_status_display,
 			placeholder($_->source_status),
-			placeholder($_->group),
+			placeholder($_->destination),
 			placeholder($fx->{params}->{$_->vol}->[0]),
 			placeholder($fx->{params}->{$_->pan}->[0]),
         } @tracks;
@@ -418,7 +435,7 @@ sub show_tracks {
 sub showlist {
 	package Audio::Nama;
 
-	my @list = grep{ ! $_->hide } Audio::Nama::Track::all();
+	my @list = grep{ ! $_->hide } Audio::Nama::all_tracks();
 	my $section = [undef,undef,@list];
 	my ($screen_lines, $columns);
 	if( $text->{term} )
@@ -434,7 +451,7 @@ sub showlist {
 		push @sections, [undef,undef, map $tn{$_},qw(Master Mixdown)];
 		push @sections, [$tn{Master},$bn{Main},map $tn{$_},$bn{Main}->tracks ];
 
-	if( $mode->{mastering} ){
+	if( $mode->mastering ){
 
 		push @sections, [undef,undef, map $tn{$_},$bn{Mastering}->tracks]
 
@@ -447,31 +464,24 @@ sub showlist {
 }
 
 
-format STDOUT_TOP =
-Track Name      Ver. Setting  Status   Source           Send        Vol  Pan 
-=============================================================================
-.
-format STDOUT =
-@>>   @<<<<<<<<< @>    @<<     @<< @|||||||||||||| @||||||||||||||  @>>  @>> ~~
-splice @{$text->{format_fields}}, 0, 9
-.
-
-
 #### Some Text Commands
 
 sub t_load_project {
 	package Audio::Nama;
 	return if engine_running() and Audio::Nama::ChainSetup::really_recording();
 	my $name = shift;
-	print "input name: $name\n";
+	pager("input name: $name\n");
 	my $newname = remove_spaces($name);
 	$newname =~ s(/$)(); # remove trailing slash
-	print("Project $newname does not exist\n"), return
+	throw("Project $newname does not exist\n"), return
 		unless -d join_path(project_root(), $newname);
 	stop_transport();
+	save_state();
 	load_project( name => $newname );
-	print "loaded project: $project->{name}\n";
+	pager("loaded project: $project->{name}\n");
+	{no warnings 'uninitialized';
 	logpkg(__FILE__,__LINE__,'debug',"load hook: $config->{execute_on_project_load}");
+	}
 	Audio::Nama::process_command($config->{execute_on_project_load});
 }
 sub t_create_project {
@@ -481,35 +491,30 @@ sub t_create_project {
 		name => remove_spaces($name),
 		create => 1,
 	);
-	print "created project: $project->{name}\n";
+	pager("created project: $project->{name}\n");
 
 }
 sub mixdown {
-	my $quiet = shift;
-	pager3("Enabling mixdown to file") if ! $quiet;
-	$tn{Mixdown}->set(rw => 'REC'); 
-	$tn{Master}->set(rw => 'OFF'); 
-	$bn{Main}->set(rw => 'REC');
+	pager_newline("Enabling mixdown to file") if ! $quiet;
+	$tn{Mixdown}->set(rw => REC); 
 }
 sub mixplay { 
-	my $quiet = shift;
-	pager3("Setting mixdown playback mode.") if ! $quiet;
-	$tn{Mixdown}->set(rw => 'MON');
-	$tn{Master}->set(rw => 'MON'); 
-	$bn{Main}->set(rw => 'OFF');
+	pager_newline("Setting mixdown playback mode.") if ! $quiet;
+	$tn{Mixdown}->set(rw => PLAY);
+	$tn{Master}->set(rw => OFF); 
+	$bn{Main}->set(rw => OFF);
 }
 sub mixoff { 
-	my $quiet = shift;
-	pager3("Leaving mixdown mode.") if ! $quiet;
-	$tn{Mixdown}->set(rw => 'OFF');
-	$tn{Master}->set(rw => 'MON'); 
-	$bn{Main}->set(rw => 'REC') if $bn{Main}->rw eq 'OFF';
+	pager_newline("Leaving mixdown mode.") if ! $quiet;
+	$tn{Mixdown}->set(rw => OFF);
+	$tn{Master}->set(rw => MON); 
+	$bn{Main}->set(rw => MON);
 }
 sub remove_fade {
 	my $i = shift;
 	my $fade = $Audio::Nama::Fade::by_index{$i}
-		or print("fade index $i not found. Aborting."), return 1;
-	print "removing fade $i from track " .$fade->track ."\n"; 
+		or throw("fade index $i not found. Aborting."), return 1;
+	pager("removing fade $i from track " .$fade->track ."\n");
 	$fade->remove;
 }
 sub import_audio {
@@ -520,14 +525,14 @@ sub import_audio {
 
 	# check that track is audible
 
-	$track->set(rw => 'MON');
+	$track->set(rw => PLAY);
 
 }
 sub destroy_current_wav {
-	carp($this_track->name.": must be set to MON."), return
-		unless $this_track->rec_status eq 'MON';
+	carp($this_track->name.": must be set to PLAY."), return
+		unless $this_track->rec_status eq PLAY;
 	$this_track->current_version or
-		say($this_track->name, 
+		throw($this_track->name, 
 			": No current version (track set to OFF?) Skipping."), return;
 	my $wav = $this_track->full_path;
 	my $reply = $text->{term}->readline("delete WAV file $wav? [n] ");
@@ -535,7 +540,7 @@ sub destroy_current_wav {
 	if ( $reply =~ /y/i ){
 		# remove version comments, if any
 		delete $this_track->{version_comment}{$this_track->current_version};
-		print "Unlinking.\n";
+		pager("Unlinking.\n");
 		unlink $wav or warn "couldn't unlink $wav: $!\n";
 		restart_wav_memoize();
 	}
@@ -558,32 +563,25 @@ sub pan_check {
 }
 
 sub remove_track_cmd {
-	my ($track, $quiet) = @_;
+	my ($track) = @_;
 	
 	# avoid having ownerless SlaveTracks.  
  	Audio::Nama::ChainSetup::remove_temporary_tracks();
- 	
-	# remove track quietly if requested
-		if ( 	! $quiet 
-			and ! $config->{quietly_remove_tracks}) 
-		{
-			my $name = $track->name; 
-			my $reply = $text->{term}->readline("remove track $name? [n] ");
-			$reply =~ /y/i or return
-			pager2( "Removing track. All WAV files will be kept.")
-		}
+		$quiet or pager( "Removing track /",$track->name,"/.  All WAV files will be kept.");
+		remove_submix_helper_tracks($track->name);
 		$track->remove;
+		$this_track = $tn{Master};
 		1
 }
 sub unity {
 	my ($track, $save_level) = @_;
 	if ($save_level){
-		$track->set(old_vol_level => params($track->vol)->[0]);
+		$track->set(old_vol_level => fxn($track->vol)->params->[0]);
 	}
 	effect_update_copp_set( 
 		$track->vol, 
 		0, 
-		$config->{unity_level}->{type($track->vol)}
+		$config->{unity_level}->{fxn($track->vol)->type}
 	);
 }
 sub vol_back {
