@@ -5,7 +5,7 @@ package Audio::Nama;
 package Audio::Nama::Track;
 use Audio::Nama::Globals qw(:all);
 use Audio::Nama::Log qw(logpkg logsub);
-
+use List::MoreUtils qw(first_index);
 # Objects belonging to Track and its subclasses
 # have a 'class' field that is set when the 
 # object is created, and used when restoring
@@ -29,12 +29,11 @@ use Memoize qw(memoize unmemoize);
 no warnings qw(uninitialized redefine);
 our $VERSION = 1.0;
 
-use Audio::Nama::Util qw(freq input_node dest_type join_path);
+use Audio::Nama::Util qw(freq input_node dest_type dest_string join_path);
 use vars qw($n %by_name @by_index %track_names %by_index);
 our @ISA = 'Audio::Nama::Wav';
 use Audio::Nama::Object qw(
 					class 			
-					was_class		
 					is_mix_track		
 					n   			
 					name
@@ -91,14 +90,11 @@ use Audio::Nama::Object qw(
 					target			
 					project			
 					
-					rec_defeat		
-
-					effect_chain_stack 
-
 					comment			
 					version_comment 
 
 					forbid_user_ops	
+					engine_group
 
 					
 					
@@ -135,18 +131,6 @@ sub idx { # return first free track index
 		return $n if not $by_index{$n}
 	}
 }
-sub all { sort{$a->n <=> $b->n } values %by_name }
-
-sub rec_hookable { grep{ $_->group ne 'Temp' and $_->group ne 'Insert' } all() }
-
-{ my %system_track = map{ $_, 1} qw( Master Mixdown Eq Low Mid High Boost );
-sub user {
-	grep{ ! $system_track{$_} } map{$_->name} all();
-}
-sub is_user_track   { !  $system_track{$_[0]->name} } 
-sub is_system_track {    $system_track{$_[0]->name} } 
-}
-
 sub new {
 	# returns a reference to an object 
 	#
@@ -174,7 +158,6 @@ sub new {
 					class	=> $class,
 					name 	=> "Audio_$n", 
 					group	=> 'Main', 
-		#			rw   	=> 'REC', # Audio::Nama::add_track() sets REC if necessary
 					n    	=> $n,
 					ops     => [],
 					width => 1,
@@ -184,24 +167,22 @@ sub new {
 					modifiers 		=> q(), # start, reverse, audioloop, playat
 					looping 		=> undef, # do we repeat our sound sample
 					source_type 	=> q(soundcard),
-					source_id   	=> 1,
+					source_id   	=> "1",
 					send_type 		=> undef,
 					send_id   		=> undef,
-					effect_chain_stack => [],
+					old_vol_level	=> undef,
 
 					@_ 			}, $class;
 
-	#print "object class: $class, object type: ", ref $object, $/;
 	$track_names{$vals{name}}++;
-	#print "names used: ", Audio::Nama::json_out( \%track_names );
 	$by_index{$n} = $object;
 	$by_name{ $object->name } = $object;
 	Audio::Nama::add_pan_control($n);
 	Audio::Nama::add_volume_control($n);
 
 	$Audio::Nama::this_track = $object;
+	$Audio::Nama::ui->track_gui($object->n) unless $object->hide;
 	$object;
-	
 }
 
 
@@ -226,7 +207,6 @@ sub full_path { my $track = shift; join_path($track->dir, $track->current_wav) }
 sub group_last {
 	my $track = shift;
 	my $bus = $bn{$track->group}; 
-	#print join " ", 'searching tracks:', $bus->tracks, $/;
 	$bus->last;
 }
 
@@ -235,10 +215,9 @@ sub last { $_[0]->versions->[-1] || 0 }
 sub current_wav {
 	my $track = shift;
 	my $last = $track->current_version;
-	#print "last found is $last\n"; 
-	if 	($track->rec_status eq 'REC'){ 
+	if 	($track->rec_status eq REC){ 
 		$track->name . '_' . $last . '.wav'
-	} elsif ( $track->rec_status eq 'MON'){ 
+	} elsif ( $track->rec_status eq PLAY){ 
 		my $filename = $track->targets->{ $track->monitor_version } ;
 		$filename
 	} else {
@@ -252,16 +231,16 @@ sub current_version {
 	my $status = $track->rec_status;
 	#logpkg(__FILE__,__LINE__,'debug', "last: $last status: $status");
 
-	# two possible version numbers, depending on REC/MON status
+	# two possible version numbers, depending on REC/PLAY status
 	
-	if 	($status eq 'REC' and ! $track->rec_defeat)
+	if 	($status eq REC)
 	{ 
 		my $last = $config->{use_group_numbering} 
 					? Audio::Nama::Bus::overall_last()
 					: $track->last;
 		return ++$last
 	}
-	elsif ( $status eq 'MON'){ return $track->monitor_version } 
+	elsif ( $status eq PLAY){ return $track->monitor_version } 
 	else { return 0 }
 }
 
@@ -271,18 +250,25 @@ sub monitor_version {
 	my $bus = $bn{$track->group};
 	return $track->version if $track->version 
 				and grep {$track->version  == $_ } @{$track->versions} ;
-	return $bus->version if $bus->version 
-				and grep {$bus->version  == $_ } @{$track->versions};
-	return undef if $bus->version;
 	$track->last;
 }
 
 sub maybe_monitor { # ordinary sub, not object method
 	my $monitor_version = shift;
-	return 'MON' if $monitor_version and ! ($mode->{preview} eq 'doodle');
-	return 'OFF';
+	return PLAY if $monitor_version and ! $mode->doodle;
+	return OFF;
 }
 
+# if you belong to a bus with an opinion, go that way
+sub engine_group {
+	my $track = shift;
+	my $bus = $bn{$track->group};
+	$bus->engine_group || $track->{engine_group} || 'Nama'
+}
+sub engine {
+	my $track = shift;
+	$en{$track->engine_group}
+}
 sub rec_status {
 #	logsub("&rec_status");
 	my $track = shift;
@@ -292,47 +278,48 @@ sub rec_status {
 
 	my $bus = $bn{$track->group};
 	#logpkg(__FILE__,__LINE__,'debug', join " ", "bus:",$bus->name, $bus->rw);
-	logpkg(__FILE__,__LINE__,'debug', "track: ", $track->name, ", source: ",
-		$track->source_id, ", monitor version: $monitor_version");
+	logpkg(__FILE__,__LINE__,'debug', "track: $track->{name}, source: $track->{source_id}, monitor version: $monitor_version");
+	#logpkg(__FILE__,__LINE__,'debug', "track: ", $track->name, ", source: ",
+	#	$track->source_id, ", monitor version: $monitor_version");
 
-	# first, check for conditions resulting in status 'OFF'
+	# first, check for conditions resulting in status OFF
 
-	if ( $bus->rw eq 'OFF'
-		or $track->rw eq 'OFF'
-		or $mode->{preview} eq 'doodle' and $track->rw eq 'REC' and 
+	if ( $bus->rw eq OFF
+		or $track->rw eq OFF
+		or $mode->doodle and ! $mode->eager and $track->rw eq REC and 
 			$setup->{tracks_with_duplicate_inputs}->{$track->name}
-	){ 	return			  'OFF' }
+		or $track->engine_group ne $Audio::Nama::this_engine->name
+	){ 	return			  OFF }
 
-	# having reached here, we know $bus->rw and $track->rw are REC or MON
-	# so the result will be REC or MON if conditions are met
+	# having reached here, we know $bus->rw and $track->rw are REC or PLAY
+	# so the result will be REC or PLAY if conditions are met
 
 	# second, set REC status if possible
 	
-	if( $track->rw eq 'REC'){
+	if( $track->rw eq REC){
 
-		given( $track->source_type){
-			when('track')		{ return 'REC' }
-			when('jack_client'){
+		my $source_type = $track->source_type;
+		if ($source_type eq 'track' or $source_type eq 'loop'){ return REC }
+		elsif ($source_type eq 'jack_client'){
 
 				# we expect an existing JACK client that
 				# *outputs* a signal for our track input
 				
 				Audio::Nama::jack_client_array($track->source_id,'output')
-					?  return 'REC'
-					:  return 'OFF'
+					?  return REC
+					:  return OFF
 			}
-			when('jack_manual')		{ return 'REC' }
-			when('jack_ports_list')	{ return 'REC' }
-			when('null')			{ return 'REC' }
-			when('soundcard')		{ return 'REC' }
-			when('bus')				{ return 'REC' } # maybe $track->rw ??
-			default 				{ return 'OFF' }
-			#default { croak $track->name. ": missing source type" }
-			# fall back to MON
-			#default {  maybe_monitor($monitor_version)  }
-		}
+		elsif ($source_type eq 'jack_manual'){ return REC }
+		elsif ($source_type eq 'jack_ports_list'){ return REC }
+		elsif ($source_type eq 'null')	{ return REC }
+		elsif ($source_type eq 'rtnull')	{ return REC }
+		elsif ($source_type eq 'soundcard'){ return REC }
+		elsif ($source_type eq 'bus')	{ return REC } # maybe $track->rw ??
+		else { return OFF }
 	}
-	# third, set MON status if possible
+	elsif( $track->rw eq MON){ 'MON' }
+
+	# set PLAY status if possible
 	
 	else { 			maybe_monitor($monitor_version)
 
@@ -341,69 +328,66 @@ sub rec_status {
 sub rec_status_display {
 	my $track = shift;
 	my $status = $track->rec_status;
-	($track->rw eq 'REC' and $track->rec_defeat) ? "($status)" : $status;
+	my $setting = $track->rw;
+	$status .= lc " ($setting)" if $status ne $setting;  
+	$status .= " v".$track->current_version if $status eq REC;
+	$status
 }
-
 # these settings will only affect WAV playback
 
 sub region_start_time {
 	my $track = shift;
-	#return if $track->rec_status ne 'MON';
-	carp $track->name, ": expected MON status" if $track->rec_status ne 'MON';
-	Audio::Nama::Mark::unadjusted_mark_time( $track->region_start )
+	#return if $track->rec_status ne PLAY;
+	carp $track->name, ": expected PLAY status" if $track->rec_status ne PLAY;
+	Audio::Nama::Mark::time_from_tag( $track->region_start )
 }
 sub region_end_time {
 	my $track = shift;
-	#return if $track->rec_status ne 'MON';
-	carp $track->name, ": expected MON status" if $track->rec_status ne 'MON';
+	#return if $track->rec_status ne PLAY;
+	carp $track->name, ": expected PLAY status" if $track->rec_status ne PLAY;
 	if ( $track->region_end eq 'END' ){
 		return $track->wav_length;
 	} else {
-		Audio::Nama::Mark::unadjusted_mark_time( $track->region_end )
+		Audio::Nama::Mark::time_from_tag( $track->region_end )
 	}
 }
 sub playat_time {
 	my $track = shift;
-	carp $track->name, ": expected MON status" if $track->rec_status ne 'MON';
-	#return if $track->rec_status ne 'MON';
-	Audio::Nama::Mark::unadjusted_mark_time( $track->playat )
+	carp $track->name, ": expected PLAY status" if $track->rec_status ne PLAY;
+	#return if $track->rec_status ne PLAY;
+	Audio::Nama::Mark::time_from_tag( $track->playat )
 }
 
 # the following methods adjust
 # region start and playat values during edit mode
 
-sub adjusted_region_start_time {
+sub shifted_region_start_time {
 	my $track = shift;
 	return $track->region_start_time unless $mode->{offset_run};
-	Audio::Nama::set_edit_vars($track);
-	Audio::Nama::new_region_start();
+	Audio::Nama::new_region_start(Audio::Nama::edit_vars($track));
 	
 }
-sub adjusted_playat_time { 
+sub shifted_playat_time { 
 	my $track = shift;
 	return $track->playat_time unless $mode->{offset_run};
-	Audio::Nama::set_edit_vars($track);
-	Audio::Nama::new_playat();
+	Audio::Nama::new_playat(Audio::Nama::edit_vars($track));
 }
-sub adjusted_region_end_time {
+sub shifted_region_end_time {
 	my $track = shift;
 	return $track->region_end_time unless $mode->{offset_run};
-	Audio::Nama::set_edit_vars($track);
-	Audio::Nama::new_region_end();
+	Audio::Nama::new_region_end(Audio::Nama::edit_vars($track));
 }
 
 sub region_is_out_of_bounds {
 	return unless $mode->{offset_run};
 	my $track = shift;
-	Audio::Nama::set_edit_vars($track);
-	Audio::Nama::case() =~ /out_of_bounds/
+	Audio::Nama::case(Audio::Nama::edit_vars($track)) =~ /out_of_bounds/
 }
 
 sub fancy_ops { # returns list 
 	my $track = shift;
-	my @skip = 	grep {Audio::Nama::fx($_)}  # must have data structure
-				grep {$_} 	   # must be non null
-				map { $track->$_ } qw(vol pan fader latency_op );
+	my @skip = 	grep {Audio::Nama::fxn($_)}  # must exist
+				map { $track->{$_} } qw(vol pan fader latency_op );
 
 	# make a dictionary of ops to exclude
 	# that includes utility ops and their controllers
@@ -412,7 +396,11 @@ sub fancy_ops { # returns list
 
 	map{ $skip{$_}++ } @skip, Audio::Nama::expanded_ops_list(@skip);
 
-	grep{ ! $skip{$_} } @{ $track->ops };
+	grep{ ! $skip{$_} } @{ $track->{ops} || [] };
+}
+sub fancy_ops_o {
+	my $track = shift;
+	map{ Audio::Nama::fxn($_) } $track->fancy_ops();
 }
 		
 sub snapshot {
@@ -422,7 +410,6 @@ sub snapshot {
 	my $i = 0;
 	for(@$fields){
 		$snap{$_} = $track->$_;
-		#say "key: $_, val: ",$track->$_;
 	}
 	\%snap;
 }
@@ -435,7 +422,9 @@ sub input_path {
 	my $track = shift;
 
 	# the corresponding bus handles input routing for mix tracks
-	return() if $track->is_mix_track and $track->rec_status eq 'REC';
+	
+	# bus mix tracks don't usually need to be connected
+	return() if $track->is_mix_track and $track->rec_status ne PLAY;
 
 	# the track may route to:
 	# + another track
@@ -444,24 +433,15 @@ sub input_path {
 
 	if($track->source_type eq 'track'){ ($track->source_id, $track->name) } 
 
-	elsif($track->rec_status eq 'REC'){ 
+	elsif($track->rec_status =~ /REC|MON/){ 
 		(input_node($track->source_type), $track->name) } 
 
-	elsif($track->rec_status eq 'MON' and $mode->{preview} ne 'doodle'){
+	elsif($track->rec_status eq PLAY and ! $mode->doodle){
 		('wav_in', $track->name) 
 	}
 }
 
 
-sub remove_effect_from_track { 
-	# doesn't touch $fx->{applied} or $fx->{params} data structures 
-	my $track = shift;
-	my @ids = @_;
-	$track->set(ops => [ grep { my $existing = $_; 
-									! grep { $existing eq $_
-									} @ids }  
-							@{$track->ops} ]);
-}
 sub has_insert  { $_[0]->prefader_insert or $_[0]->postfader_insert }
 
 sub prefader_insert { Audio::Nama::Insert::get_id($_[0],'pre') }
@@ -478,7 +458,6 @@ sub remove {
 	my $track = shift;
 	my $n = $track->n;
 	$ui->remove_track_gui($n); 
- 	$Audio::Nama::this_track = $ti{Audio::Nama::Track::idx() - 1};
 	# remove corresponding fades
 	map{ $_->remove } grep { $_->track eq $track->name } values %Audio::Nama::Fade::by_index;
 	# remove effects
@@ -487,9 +466,6 @@ sub remove {
  	delete $by_name{$track->name};
 }
 
-	
-	
-
 ### object methods for text-based commands 
 
 # Reasonable behavior whether 'source' and 'send' commands 
@@ -525,70 +501,45 @@ sub set_io {
 	# set values, returning new setting
 	$type ||= dest_type( $id );
 	
-	given ($type){
+	if( $type eq 'track')		{}
+	elsif( $type eq 'soundcard'){} # no changes needed 
+	elsif( $type eq 'bus')     	{} # -ditto-
+	#elsif( $type eq 'loop')    {}  # unused at present
+
+	# don't allow user to set JACK I/O unless JACK server is running
 	
-		
-		when('track'){}
-		when('soundcard'){} # no changes needed 
-		when ('bus')     {} # -ditto-
-		#when('loop')     {}  # unused at present
+	elsif( $type =~ /jack/ ){
+		Audio::Nama::throw("JACK server not running! "
+			,"Cannot set JACK client or port as track source."), 
+				return unless $jack->{jackd_running};
 
-		# rec_defeat tracks with 'null' input
-
-		when ('null'){ 
-
-			if ( $direction eq 'source' ){
-				$track->set(rec_defeat => 1);
-				say $track->name, ": recording disabled by default for 'null' input.";
-				say "Use 'rec_enable' if necessary";
-			}
-		}
-		when ('rtnull'){ 
-
-			if ( $direction eq 'source' ){
-				$track->set(rec_defeat => 1);
-				say $track->name, ": recording disabled by default for 'rtnull' input.";
-				say "Use 'rec_enable' if necessary";
-			}
-		}
-
-		# don't allow user to set JACK I/O unless JACK server is running
-		
- 		when ( /jack/ ){
-			say("JACK server not running! "
-				,"Cannot set JACK client or port as track source."), 
-					return unless $jack->{jackd_running};
-
-			continue;
-		} 
-
-		when ('jack_manual'){
+		if( $type eq 'jack_manual'){
 
 			my $port_name = $track->jack_manual_port($direction);
 
- 			say $track->name, ": JACK $direction port is $port_name. Make connections manually.";
+			Audio::Nama::pager($track->name, ": JACK $direction port is $port_name. Make connections manually.");
 			$id = 'manual';
 			$id = $port_name;
 			$type = 'jack_manual';
 		}
-		when ('jack_client'){
+		elsif( $type eq 'jack_client'){
 			my $client_direction = $direction eq 'source' ? 'output' : 'input';
 
 			my $name = $track->name;
 			my $width = scalar @{ Audio::Nama::jack_client_array($id, $client_direction) };
-			$width or say 
-				qq($name: $direction port for JACK client "$id" not found.);
+			$width or Audio::Nama::pager(
+				qq($name: $direction port for JACK client "$id" not found.));
 			$width or return;
-			$width ne $track->width and say 
+			$width ne $track->width and Audio::Nama::pager(
 				$track->name, ": track set to ", Audio::Nama::width($track->width),
-				qq(, but JACK source "$id" is ), Audio::Nama::width($width), '.';
+				qq(, but JACK source "$id" is ), Audio::Nama::width($width), '.');
 		}
-		when( 'jack_ports_list' ){
+		elsif( $type eq 'jack_ports_list' ){
 			$id =~ /(\w+)\.ports/;
 			my $ports_file_name = ($1 || $track->name) .  '.ports';
 			$id = $ports_file_name;
 			# warn if ports do not exist
-			say($track->name, qq(: ports file "$id" not found in ),Audio::Nama::project_root(),". Skipping."), 
+			Audio::Nama::throw($track->name, qq(: ports file "$id" not found in ),Audio::Nama::project_root(),". Skipping."), 
 				return unless -e join_path( Audio::Nama::project_root(), $id );
 			# check if ports file parses
 		}
@@ -614,13 +565,9 @@ sub set_source {
 	my $new_source = $track->input_object_text;;
 	my $object = $new_source;
 	if ( $old_source  eq $new_source ){
-		Audio::Nama::pager2($track->name, ": input unchanged, $object");
+		Audio::Nama::pager($track->name, ": input unchanged, $object");
 	} else {
-		Audio::Nama::pager2("Track ",$track->name, ": source set to $object");
-		Audio::Nama::pager2("Track ",$track->name, ": record enabled"), 
-			$track->set(rec_defeat => 0) 
-			if transition_from_null($old_source,$new_source)
-			and $track->rec_defeat;
+		Audio::Nama::pager("Track ",$track->name, ": source set to $object");
 	}
 }
 {
@@ -640,13 +587,13 @@ sub set_version {
 	my ($track, $n) = @_;
 	my $name = $track->name;
 	if ($n == 0){
-		print "$name: following latest version\n";
+		Audio::Nama::pager("$name: following bus default\n");
 		$track->set(version => $n)
 	} elsif ( grep{ $n == $_ } @{$track->versions} ){
-		print "$name: anchoring version $n\n";
+		Audio::Nama::pager("$name: anchoring version $n\n");
 		$track->set(version => $n)
 	} else { 
-		print "$name: version $n does not exist, skipping.\n"
+		Audio::Nama::throw("$name: version $n does not exist, skipping.\n")
 	}
 }
 
@@ -660,10 +607,10 @@ sub set_send {
 	logpkg(__FILE__,__LINE__,'debug', "send is now $new_send");
 	my $object = $track->output_object_text;
 	if ( $old_send  eq $new_send ){
-		Audio::Nama::pager2("Track ",$track->name, ": send unchanged, ",
+		Audio::Nama::pager("Track ",$track->name, ": send unchanged, ",
 			( $object ?  $object : 'off'));
 	} else {
-		Audio::Nama::pager2("Track ",$track->name, ": ", 
+		Audio::Nama::pager("Track ",$track->name, ": ", 
 		$object 
 			? "$object is now a send target" 
 			: "send target is turned off.");
@@ -700,14 +647,43 @@ sub output_object_text {   # text for user display
 	$track->object_as_text('send');
 
 }
+sub bus_name { 
+	my $track = shift;
+	return unless $track->is_mix_track;
+	$track->name eq 'Master' 
+		? 'Main'
+		: $track->name
+}
 sub source_status {
 	my $track = shift;
-	my $id = $track->source_id;
-	return unless $id;
-	$track->rec_status eq 'REC' ? $id : "[$id]"
+	return $track->current_wav if $track->rec_status eq PLAY ;
+	#return $track->name eq 'Master' ? $track->bus_name : '' if $track->is_mix_track;
+	return $track->bus_name . " bus" if $track->is_mix_track;
+	return $track->source_id unless $track->source_type eq 'soundcard';
+	my $ch = $track->source_id;
+	my @channels;
+	push @channels, $_ for $ch .. ($ch + $track->width - 1);
+	join '/', @channels
+}
+sub destination {
+	my $track = shift;
+	# display logic 
+	# always show the bus
+	# except for tracks that belongs to the bus null.
+	# in that case, show the specific source.
+	#
+	# for these mix tracks, we use the
+	# track's own send_type/send_id
 	
+	my $out;
+	$out .= $track->group unless $track->group =~ /^(null|Master)$/;
+	my $send_id = $track->send_id;
+	my $send_type = $track->send_type;
+	return $out if ! $send_type;
+	$out .=	', ' if $out;
+	$out .= dest_string($send_type, $send_id, $track->width);
+	$out
 }
-
 sub set_rec {
 	my $track = shift;
 	if (my $t = $track->target){
@@ -717,41 +693,31 @@ sub set_rec {
 				if $track->project;
 			$msg .= qq(.\n);
 			$msg .= "Can't set a track alias to REC.\n";
-		print $msg;
+		Audio::Nama::throw($msg);
 		return;
 	}
-	$track->set_rw('REC');
+	$track->set_rw(REC);
+}
+sub set_play {
+	my $track = shift;
+	$track->set_rw(PLAY);
 }
 sub set_mon {
 	my $track = shift;
-	$track->set_rw('MON');
+	$track->set_rw(MON);
 }
 sub set_off {
 	my $track = shift;
-	$track->set_rw('OFF');
+	$track->set_rw(OFF);
 }
 
-=comment
-mix
-self bus      brothers
-REC  MON 
-MON  OFF
-OFF  OFF
-
-member
-REC  REC      REC->MON
-MON  OFF->MON REC/MON->OFF
-OFF  --       --
-
-=cut
-	
 sub set_rw {
 	my ($track, $setting) = @_;
 	#my $already = $track->rw eq $setting ? " already" : "";
 	$track->set(rw => $setting);
 	my $status = $track->rec_status();
-	say $track->name, " set to $setting", 
-		($status ne $setting ? ", but current status is $status" : "");
+	Audio::Nama::pager($track->name, " set to $setting", 
+		($status ne $setting ? ", but current status is $status" : ""));
 
 }
 	
@@ -760,26 +726,26 @@ sub set_rw {
 
 sub normalize {
 	my $track = shift;
-	if ($track->rec_status ne 'MON'){
-		print $track->name, ": You must set track to MON before normalizing, skipping.\n";
+	if ($track->rec_status ne PLAY){
+		Audio::Nama::throw($track->name, ": You must set track to PLAY before normalizing, skipping.\n");
 		return;
 	} 
-	# track version will exist if MON status
+	# track version will exist if PLAY status
 	my $cmd = 'ecanormalize ';
 	$cmd .= $track->full_path;
-	print "executing: $cmd\n";
+	Audio::Nama::pager("executing: $cmd\n");
 	system $cmd;
 }
 sub fixdc {
 	my $track = shift;
-	if ($track->rec_status ne 'MON'){
-		print $track->name, ": You must set track to MON before fixing dc level, skipping.\n";
+	if ($track->rec_status ne PLAY){
+		Audio::Nama::throw($track->name, ": You must set track to PLAY before fixing dc level, skipping.\n");
 		return;
 	} 
 
 	my $cmd = 'ecafixdc ';
 	$cmd .= $track->full_path;
-	print "executing: $cmd\n";
+	Audio::Nama::pager("executing: $cmd\n");
 	system $cmd;
 }
 sub wav_length {
@@ -842,32 +808,34 @@ sub import_audio  {
 	my $track = shift;
 	my ($path, $frequency) = @_; 
 	$path = Audio::Nama::expand_tilde($path);
-	#say "path: $path";
 	my $version  = $track->last + 1;
 	if ( ! -r $path ){
-		print "$path: non-existent or unreadable file. No action.\n";
+		Audio::Nama::throw("$path: non-existent or unreadable file. No action.\n");
 		return;
 	}
 	my ($depth,$width,$freq) = split ',', Audio::Nama::wav_format($path);
-	say "format: ", Audio::Nama::wav_format($path);
+	Audio::Nama::pager_newline("format: ", Audio::Nama::wav_format($path));
 	$frequency ||= $freq;
 	if ( ! $frequency ){
-		say "Cannot detect sample rate of $path. Skipping.";
-		say "Use 'import_audio <path> <frequency>' if possible.";
+		Audio::Nama::throw("Cannot detect sample rate of $path. Skipping.",
+		"Maybe 'import_audio <path> <frequency>' will help.");
 		return 
 	}
 	my $desired_frequency = freq( $config->{raw_to_disk_format} );
 	my $destination = join_path(Audio::Nama::this_wav_dir(),$track->name."_$version.wav");
-	#say "destination: $destination";
 	if ( $frequency == $desired_frequency and $path =~ /.wav$/i){
-		say "copying $path to $destination";
+		Audio::Nama::pager_newline("copying $path to $destination");
 		copy($path, $destination) or die "copy failed: $!";
 	} else {	
 		my $format = Audio::Nama::signal_format($config->{raw_to_disk_format}, $width);
-		say "importing $path as $destination, converting to $format";
-		my $cmd = qq(ecasound -f:$format -i:resample-hq,$frequency,"$path" -o:$destination);
-		#say $cmd;
-		system($cmd) == 0 or say("Ecasound exited with error: ", $?>>8), return;
+		Audio::Nama::pager_newline("importing $path as $destination, converting to $format");
+		Audio::Nama::teardown_engine();
+		my $ecs = qq(-f:$format -i:resample-hq,$frequency,"$path" -o:$destination);
+		my $path = join_path(Audio::Nama::project_dir()."convert.ecs");
+		write_file($path, $ecs);
+		load_ecs($path) or Audio::Nama::throw("$path: load failed, aborting"), return;
+		eval_iam('start');
+		Audio::Nama::sleeper(0.2); sleep 1 while Audio::Nama::engine_running();
 	} 
 	Audio::Nama::restart_wav_memoize() if $config->{opts}->{R}; # usually handled by reconfigure_engine() 
 }
@@ -892,56 +860,44 @@ sub version_has_edits {
      		and $_->host_version == $track->monitor_version
 		} values %Audio::Nama::Edit::by_name;
 }	
-#### UNUSED 
-sub edits_enabled {
+sub op { $project->{current_op}->{$_[0]->name} //= $_[0]->{ops}->[-1] }
+
+sub param { $project->{current_param}->{$_[0]->op} //= 1 }
+
+sub stepsize {
+	$project->{current_stepsize}->{$_[0]->op}->[$_[0]->param] //= 0.01 
+	# TODO use hint if available
+}
+sub pos {
 	my $track = shift;
-	my $bus;
-	$bus = $bn{$track->name}
-	and $bus->rw ne 'OFF'
-	and $track->rec_status eq 'REC' 
-	and $track->rec_defeat
-	and $track->is_mix_track
+	first_index{$_ eq $track->op} @{$track->ops};
 }
-##### 
 
 sub set_track_class {
 	my ($track, $class) = @_;
 	bless $track, $class;
 	$track->set(class => $class);
 }
-
-
 sub busify {
-
-	# does not set an existing bus to REC or MON!
-	
 	my $track = shift;
-	my $name = $track->name;
-
-	# create the bus if needed
-	# create or convert named track to mix track
-	
-	Audio::Nama::add_sub_bus($name) unless $track->is_system_track;
-
+	Audio::Nama::add_sub_bus($track->name) unless $track->is_system_track;
 }
 sub unbusify {
 	my $track = shift;
 	return if $track->is_system_track;
-	$track->set( rw => 'MON',
-                 rec_defeat => 0);
-	$track->set_track_class($track->was_class // 'Audio::Nama::Track');
+	$track->set( rw => PLAY);
 }
 
-sub adjusted_length {
+sub shifted_length {
 	my $track = shift;
 	my $setup_length;
 	if ($track->region_start){
-		$setup_length = 	$track->adjusted_region_end_time
-				  - $track->adjusted_region_start_time
+		$setup_length = 	$track->shifted_region_end_time
+				  - $track->shifted_region_start_time
 	} else {
 		$setup_length = 	$track->wav_length;
 	}
-	$setup_length += $track->adjusted_playat_time;
+	$setup_length += $track->shifted_playat_time;
 }
 
 sub version_comment {
@@ -954,16 +910,13 @@ sub version_comment {
 	"$v: $text\n" if $text;
 }
 # Modified from Object.p to save class
+# should this be used in other classes?
 sub as_hash {
 	my $self = shift;
 	my $class = ref $self;
 	bless $self, 'HASH'; # easy magic
-	#print json_out $self; return;
 	my %guts = %{ $self };
 	$guts{class} = $class; # make sure we save the correct class name
-	#print join " ", %guts; return;
-	#my @keys = keys %guts;
-	#map{ $output->{$_} or $output->{$_} = '~'   } @keys; 
 	bless $self, $class; # restore
 	return \%guts;
 }
@@ -1015,19 +968,19 @@ sub show_version_comments {
 }
 sub add_version_comment {
 	my ($t,$v,$text) = @_;
-	$t->targets->{$v} or say("$v: no such version"), return;	
+	$t->targets->{$v} or Audio::Nama::throw("$v: no such version"), return;	
 	$project->{track_version_comments}->{$t->name}{$v}{user} = $text;
 	$t->version_comment($v);
 }
 sub add_system_version_comment {
 	my ($t,$v,$text) = @_;
-	$t->targets->{$v} or say("$v: no such version"), return;	
+	$t->targets->{$v} or Audio::Nama::throw("$v: no such version"), return;	
 	$project->{track_version_comments}{$t->name}{$v}{system} = $text;
 	$t->version_comment($v);
 }
 sub remove_version_comment {
 	my ($t,$v) = @_;
-	$t->targets->{$v} or say("$v: no such version"), return;	
+	$t->targets->{$v} or Audio::Nama::throw("$v: no such version"), return;	
 	delete $project->{track_version_comments}{$t->name}{$v}{user};
 	$t->version_comment($v) || "$v: [comment deleted]\n";
 }
@@ -1044,8 +997,75 @@ sub rec_cleanup_script {
 	my $track = shift;
 	join_path(Audio::Nama::project_dir(), $track->name."-rec-cleanup.sh")
 }
-	
+sub is_region { defined $_[0]->{region_start} }
 
+sub current_edit { $_[0]->{current_edit}//={} }
+
+sub first_effect_of_type {
+	my $track = shift;
+	my $type = shift;
+	for my $op ( @{$track->ops} ){
+		my $FX = Audio::Nama::fxn($op);
+		return $FX if $FX->type =~ /$type/ # Plate matches el:Plate
+	}
+}
+sub is_mix_track {
+	my $track = shift;
+	($bn{$track->name} or $track->name eq 'Master') and $track->rw eq MON
+}
+sub bus { $bn{$_[0]->group} }
+
+sub effect_id_by_name {
+	my $track = shift;
+	my $ident = shift;
+	for my $FX (map{Audio::Nama::fxn($_)}@{$track->ops})
+	{ return $FX->id if $FX->name eq $ident }
+}
+sub effect_nickname_count {
+	my ($track, $nick) = @_;
+	my $count = 0;
+	for my $FX ($track->fancy_ops_o){ $count++ if $FX->name =~ /^$nick\d*$/ }
+	$count
+}
+sub unique_surname {
+	my ($track, $surname) = @_;
+	my $i = 0;
+	my @found;
+	for my $FX ($track->fancy_ops_o)
+	{ 
+		if( $FX->surname =~ /^$surname(\d*)$/)
+		{
+			push @found, $FX->surname; 
+			$i = $1 if $1 and $1 > $i
+		}
+	}
+	$surname. (@found ? ++$i : ""), join ' ',@found
+}
+sub unique_nickname {
+	my ($track, $nickname) = @_;
+	my $i = 0;
+	my @found;
+	for my $FX ($track->fancy_ops_o)
+	{ 
+		if( $FX->name =~ /^$nickname(\d*)$/)
+		{
+			push @found, $FX->name; 
+			$i = $1 if $1 and $1 > $i
+		}
+	}
+	$nickname. (@found ? ++$i : ""), "@found"
+}
+sub with_surname {
+	my ($track, $surname) = @_;
+	my @found;
+	for my $FX ($track->fancy_ops_o)
+	{ push @found, $FX->id if $FX->surname eq $surname }
+	@found ? "@found" : undef
+}
+{ my %system_track = map{ $_, 1} qw( Master Mixdown Eq Low Mid High Boost );
+sub is_user_track { ! $system_track{$_[0]->name} }
+sub is_system_track { $system_track{$_[0]->name} } 
+}
 } # end package
 
 # subclasses
@@ -1055,10 +1075,18 @@ sub rec_cleanup_script {
 package Audio::Nama::SimpleTrack; # used for Master track
 use Audio::Nama::Globals qw(:all);
 use Modern::Perl; use Carp; use Audio::Nama::Log qw(logpkg);
+use SUPER;
 no warnings qw(uninitialized redefine);
 our @ISA = 'Audio::Nama::Track';
-sub rec_status { $_[0]->rw ne 'OFF' ? 'REC' : 'OFF' }
-#sub rec_status_display { $_[0]->rw ne 'OFF' ? 'MON' : 'OFF' }
+sub rec_status {
+	my $track = shift;
+ 	$track->rw ne OFF ? 'MON' : 'OFF' 
+}
+sub destination {
+	my $track = shift; 
+	$track->SUPER() if $track->rec_status ne OFF
+}
+#sub rec_status_display { $_[0]->rw ne OFF ? 'PLAY' : 'OFF' }
 sub busify {}
 sub unbusify {}
 }
@@ -1071,13 +1099,31 @@ our @ISA = 'Audio::Nama::SimpleTrack';
 
 sub rec_status{
 	my $track = shift;
-	$mode->{mastering} ? 'MON' :  'OFF';
+ 	return OFF if $track->engine_group ne $this_engine->name;
+	$mode->{mastering} ? MON :  'OFF';
 }
 sub source_status {}
 sub group_last {0}
 sub version {0}
 }
 {
+package Audio::Nama::EarTrack; # for submix helper tracks
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Util qw(dest_string);
+use Modern::Perl; use Audio::Nama::Log qw(logpkg);
+use SUPER;
+no warnings qw(uninitialized redefine);
+our @ISA = 'Audio::Nama::SlaveTrack';
+sub destination {
+	my $track = shift;
+	my $bus = $track->bus;
+	dest_string($bus->send_type,$bus->send_id, $track->width);
+}
+sub source_status { $_[0]->target }
+sub rec_status { $_[0]->{rw} }
+sub width { $_[0]->{width} }
+}
+{
 package Audio::Nama::SlaveTrack; # for instrument monitor bus
 use Audio::Nama::Globals qw(:all);
 use Modern::Perl; use Audio::Nama::Log qw(logpkg);
@@ -1102,7 +1148,7 @@ no warnings qw(uninitialized redefine);
 our @ISA = 'Audio::Nama::SlaveTrack';
 sub rec_status{
 	my $track = shift;
-	$mode->{mastering} ? 'MON' :  'OFF';
+	$mode->{mastering} ? MON :  'OFF';
 }
 }
 {
@@ -1114,8 +1160,8 @@ sub current_version {
 	my $track = shift;
 	my $target = $tn{$track->target};
 		$target->last + 1
-# 	if ($target->rec_status eq 'MON'
-# 		or $target->rec_status eq 'REC' and $bn{$track->target}){
+# 	if ($target->rec_status eq PLAY
+# 		or $target->rec_status eq REC and $bn{$track->target}){
 # 	}
 }
 sub current_wav {
@@ -1128,19 +1174,30 @@ sub full_path { my $track = shift; Audio::Nama::join_path( $track->dir, $track->
 package Audio::Nama::MixDownTrack; 
 use Audio::Nama::Globals qw(:all);
 use Audio::Nama::Log qw(logpkg);
+use SUPER;
 our @ISA = qw(Audio::Nama::Track);
 sub current_version {	
 	my $track = shift;
 	my $last = $track->last;
 	my $status = $track->rec_status;
 	#logpkg(__FILE__,__LINE__,'debug', "last: $last status: $status");
-	if 	($status eq 'REC'){ return ++$last}
-	elsif ( $status eq 'MON'){ return $track->monitor_version } 
+	if 	($status eq REC){ return ++$last}
+	elsif ( $status eq PLAY){ return $track->monitor_version } 
 	else { return 0 }
 }
+sub source_status { 
+	my $track = shift; 
+	return 'Master' if $track->rec_status eq REC;
+	my $super = $track->super('source_status');
+	$super->($track)
+}
+sub destination {
+	my $track = shift; 
+	$tn{Master}->destination if $track->rec_status eq PLAY
+}
 sub rec_status {
 	my $track = shift;
-	return 'REC' if $track->rw eq 'REC';
+	return REC if $track->rw eq 'REC';
 	Audio::Nama::Track::rec_status($track);
 }
 sub forbid_user_ops { 1 }
@@ -1164,8 +1221,8 @@ sub current_version {
 	my $last = $track->last;
 	my $status = $track->rec_status;
 	#logpkg(__FILE__,__LINE__,'debug', "last: $last status: $status");
-	if 	($status eq 'REC' and ! $track->rec_defeat){ return ++$last}
-	elsif ( $status eq 'MON'){ return $track->monitor_version } 
+	if 	($status eq REC){ return ++$last}
+	elsif ( $status eq PLAY){ return $track->monitor_version } 
 	else { return 0 }
 }
 sub playat_time {
@@ -1181,13 +1238,94 @@ our @ISA ='Audio::Nama::Track';
 sub set_version {}
 sub versions { [$_[0]->version] }
 }
+{
+package Audio::Nama::Clip;
 
+# Clips are the units of audio used to 
+#  to make sequences. 
+
+# A clip is created from a track. Clips extend the Track
+# class in providing a position which derives from the
+# object's ordinal position in an array (clips attribute) of
+# the parent sequence object.
+ 
+# Clips differ from tracks in that clips
+# their one-based position (index) in the sequence items array.
+# index is one-based.
+
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Log qw(logpkg);
+our @ISA = qw( Audio::Nama::VersionTrack Audio::Nama::Track );
+
+sub sequence { my $self = shift; $Audio::Nama::bn{$self->group} };
+
+sub index { my $self = shift; my $i = 0;
+	for( @{$self->sequence->items} ){
+		$i++;
+		return $i if $self->name eq $_
+	}
+}
+sub predecessor {
+	my $self = shift;
+	$self->sequence->clip($self->index - 1)
+}
+sub duration {
+	my $self = shift;
+	$self->{duration} 
+		? Audio::Nama::Mark::duration_from_tag($self->{duration})
+		: $self->is_region 
+			? $self->region_end_time - $self->region_start_time 
+			: $self->wav_length;
+}
+sub endpoint { 
+	my $self = shift;
+	$self->duration + ( $self->predecessor ?  $self->predecessor->endpoint : 0 )
+}
+sub playat_time {
+	my $self = shift;
+	my $previous = $self->predecessor;
+	$previous ? $previous->endpoint : 0
+}
+
+# we currently are not compatible with offset run mode
+# perhaps we can enforce OFF status for clips under 
+# offset run mode
+
+} # end package
+{ 
+package Audio::Nama::Spacer;
+our @ISA = 'Audio::Nama::Clip';
+use SUPER;
+use Audio::Nama::Object qw(duration);
+sub rec_status { OFF }
+sub new { 
+	my ($class,%args) = @_;
+
+	# remove args we will process
+	my $duration = delete $args{duration};
+
+	# give the remainder to the superclass constructor
+	@_ = ($class, %args);
+	my $self = super();
+	#logpkg(__FILE__,__LINE__,'debug',"new object: ", json_out($self->as_hash));
+	#logpkg(__FILE__,__LINE__,'debug', "items: ",json_out($items));
+
+	# set the args removed above
+	$self->{duration} = $duration;
+	$self;
+}
+} # end package
+{ 
+package Audio::Nama::WetTrack; # for inserts
+use Audio::Nama::Globals qw(:all);
+use Modern::Perl; use Audio::Nama::Log qw(logpkg);
+our @ISA = 'Audio::Nama::SlaveTrack';
+}
 
 # ----------- Track_subs -------------
 {
 package Audio::Nama;
 use Modern::Perl;
-use Audio::Nama::Effects qw(:all);
 
 # usual track
 
@@ -1201,13 +1339,12 @@ sub add_track {
 	{ no warnings 'uninitialized';	
 	logpkg(__FILE__,__LINE__,'debug', "name: $name, ch_r: $gui->{_chr}, ch_m: $gui->{_chm}");
 	}	
-	say("$name: track name already in use. Skipping."), return 
+	Audio::Nama::throw("$name: track name already in use. Skipping."), return 
 		if $tn{$name};
-	say("$name: reserved track name. Skipping"), return
+	Audio::Nama::throw("$name: reserved track name. Skipping"), return
 	 	if grep $name eq $_, @{$mastering->{track_names}}; 
 
 	# in order to increment serially
-
 	Audio::Nama::ChainSetup::remove_temporary_tracks();
 
 	my $track = $class->new(%vals);
@@ -1217,18 +1354,18 @@ sub add_track {
 #		$track->send($gui->{_chm}) if $gui->{_chm};
 
 	my $bus = $bn{$track->group}; 
-	process_command('for mon; mon') if $mode->{preview} and $bus->rw eq 'MON';
-	$bus->set(rw => 'REC') unless $track->target; # not if is alias
+	process_command('for mon; mon') if $mode->{preview} and $bus->rw eq MON;
+	# TODO ???
+	$bus->set(rw => MON) unless $track->target; # not if is alias
 
-	# normal tracks default to 'REC'
-	# track aliases default to 'MON'
+	# normal tracks default to MON
+	# track aliases default to PLAY
 	$track->set(rw => $track->target
-					?  'MON'
-					:  'REC') ;
+					?  PLAY
+					:  $config->{new_track_rw} || MON );
 	$gui->{_track_name} = $gui->{_chm} = $gui->{_chr} = undef;
 
 	set_current_bus();
-	$ui->track_gui($track->n);
 	logpkg(__FILE__,__LINE__,'debug', "Added new track!\n", sub{$track->dump});
 	$track;
 }
@@ -1248,15 +1385,16 @@ sub add_track_alias {
 
 sub add_track_alias_project {
 	my ($name, $track, $project_name) = @_;
+	$project_name //= $Audio::Nama::project->{name}; 
 	my $dir =  join_path(project_root(), $project_name, '.wav'); 
 	if ( -d $dir ){
 		if ( glob "$dir/$track*.wav"){
-			print "Found target WAV files.\n";
+			Audio::Nama::pager("Found target WAV files.\n");
 			my @params = (target => $track, project => $project_name);
 			add_track( $name, @params );
-		} else { print "No WAV files found.  Skipping.\n"; return; }
+		} else { Audio::Nama::throw("$project_name:$track - No WAV files found.  Skipping.\n"), return; }
 	} else { 
-		print("$project_name: project does not exist.  Skipping.\n");
+		Audio::Nama::throw("$project_name: project does not exist.  Skipping.\n");
 		return;
 	}
 }
@@ -1333,7 +1471,72 @@ sub add_pan_control {
 	$ti{$n}->set(pan => $pan_id);  # save the id for next time
 	$pan_id;
 }
+sub rename_track {
+	use Cwd;
+	use File::Slurp;
+	my ($oldname, $newname, $statefile, $dir) = @_;
+	save_state();
+	my $old_dir = cwd();
+	chdir $dir;
+
+	# rename audio files
+	
+	qx(rename 's/^$oldname(?=[_.])/$newname/' *.wav);
+
+
+	# rename in State.json when candidate key
+	# is part of the specified set and the value 
+	# exactly matches $oldname
+	
+	my $state = read_file($statefile);
+
+	$state =~ s/
+		"					# open quote
+		(track| 		# one of specified fields
+		name| 
+		group| 
+		source| 
+		send_id| 
+		target| 
+		current_edit| 
+		send_id| 
+		return_id| 
+		wet_track| 
+		dry_track| 
+		track| 
+		host_track)
+		"				# close quote
+		\ 				# space
+		:				# colon
+		\ 				# space
+		"$oldname"/"$1" : "$newname"/gx;
+
+	write_file($statefile, $state);
+	my $msg = "Rename track $oldname -> $newname";
+	git_commit($msg);
+	Audio::Nama::pager($msg);
+	load_project(name => $Audio::Nama::project->{name});
+}
+sub user_tracks_present {
+	my $i = 0;
+	$i++ for user_tracks();
+	$i
+}
+sub all_tracks { sort{$a->n <=> $b->n } values %Audio::Nama::Track::by_name }
+sub audio_tracks { grep { $_->class !~ /Midi/ } all_tracks() }
+sub midi_tracks { grep { $_->class =~ /Midi/ } all_tracks() }
+sub rec_hookable_tracks { 
+	grep{ $_->group ne 'Temp' and $_->group ne 'Insert' } all_tracks() 
+}
+sub user_tracks { grep { ! $_->is_system_track } all_tracks() }
+sub system_tracks { grep { $_->is_system_track } all_tracks() }
 } # end package
+{
+package Audio::Nama::MidiTrack; 
+use Audio::Nama::Globals qw(:all);
+use Audio::Nama::Log qw(logpkg);
+our @ISA = qw(Audio::Nama::Track);
+}
 
 1;
 __END__
\ No newline at end of file
